@@ -1,23 +1,23 @@
 #' readConfigurationsFile
 #'
-#' \code{readConfigurationsFile} reads a set of target algorithms configurations 
+#' `readConfigurationsFile` reads a set of target algorithms configurations 
 #'  from a file and puts them in \pkg{irace} format. The configurations are checked 
 #'  to match the parameters description provided.
 #' 
-#' @param filename A filename from which the configurations should be read.
-#' @param parameters List of target algorithm parameters in the \pkg{irace} format.
-#' @param debugLevel Level of debug. Default: 0.
-#' @param text (optional) Character string: if file is not supplied and this is,
-#'  then parameters are read from the value of text via a text connection.
+#' @param filename (`character(1)`) \cr Filename from which the configurations should be read.
+#' @template arg_parameters
+#' @template arg_debuglevel
+#' @template arg_text
 #' 
 #' @return A data frame containing the obtained configurations. 
 #'   Each row of the data frame is a candidate configuration, 
-#'   the columns correspond to the parameter names in \code{parameters}.
+#'   the columns correspond to the parameter names in `parameters`.
 #'
 #' @seealso 
 #'   \code{\link{readParameters}} to obtain a valid parameter structure from a parameters list.
 #' 
 #' @author Manuel López-Ibáñez and Jérémie Dubois-Lacoste
+#' @md
 #' @export
 ## Read some configurations from a file.
 ## Example of an input file,
@@ -49,9 +49,9 @@ readConfigurationsFile <- function(filename, parameters, debugLevel = 0, text)
   irace.assert(is.data.frame(configurationTable))
   nbConfigurations <- nrow(configurationTable)
   # Print the table that has been read.
+  cat("# Read ", nbConfigurations, " configuration(s) from file '", filename, "'\n", sep="")
   if (debugLevel >= 2) {
-    cat("# Read ", nbConfigurations, " configurations from file '", filename, "'\n", sep="")
-    print(as.data.frame(configurationTable, stringAsFactor = FALSE))
+    print(as.data.frame(configurationTable, stringAsFactor = FALSE), digits=15)
   }
 
   namesParameters <- names(parameters$conditions)
@@ -180,14 +180,19 @@ checkForbidden <- function(configurations, forbidden)
 compile.forbidden <- function(x)
 {
   if (is.bytecode(x)) return(x)
+  # If we are given an expression, it must be a single one.
+  irace.assert(is.language(x) && (!is.expression(x) || length(x) == 1))
+  if (is.expression(x)) x <- x[[1]]
   # When a is NA and we check a == 5, we would get NA, which is
   # always FALSE, when we actually want to be TRUE, so we test
   # is.na() first below.
   
   # We expect that there will be undefined variables, since the expressions
   # will be evaluated within a data.frame later.
-  return(compiler::compile(substitute(is.na(x) | !(x), list(x = x)),
-                           options = list(suppressUndefined=TRUE)))
+  exp <- compiler::compile(substitute(is.na(x) | !(x), list(x = x)),
+                           options = list(suppressUndefined=TRUE))
+  attr(exp, "source") <- as.character(as.expression(x))
+  return(exp)
 }
 
 readForbiddenFile <- function(filename)
@@ -221,27 +226,25 @@ buildForbiddenExp <- function(configurations, parameters)
                paste0("(", pnames[has.value]," == ", values, ")", collapse = "&"))
   }
   exps <- parse(text = lines)
-  # FIXME: We should save exps somewhere for verification
   # print(exps)
   return(sapply(exps, compile.forbidden))
 }
 
 #' readScenario
 #'
-#' \code{readScenario} reads the scenario to be used by 
-#' \pkg{irace} from a file.
+#' `readScenario` reads from a file the scenario settings to be used by
+#' \pkg{irace}..
 #' 
-#' @param filename A filename from which the scenario will be
-#'     read. If empty, the default \code{scenarioFile} is used.
-#'     An example scenario file is provided in
-#'     \code{system.file(package="irace", "templates/scenario.txt.tmpl")}.
-#' @param scenario A list where tagged elements correspond to scenario
-#'     settings for \pkg{irace}. This is an initial scenario that is
-#'     overwritten for every parameter specified in the file to be read.
-#' 
-#' @return The scenario list read from the file. The scenario parameter not
-#'   present in the file are not present in the list, that is, they are
-#'   \code{NULL}.
+#' @param filename (`character(1)`) \cr Filename from which the scenario will
+#'   be read. If empty, the default `scenarioFile` is used.  An example
+#'   scenario file is provided in `system.file(``package="irace",`
+#'   `"templates/scenario.txt.tmpl")`.
+#' @templateVar arg_appendix This is an initial scenario that is overwritten
+#'   for every setting specified in the file to be read.
+#' @template arg_scenario
+#'  
+#' @return The scenario list read from the file. The scenario settings not
+#'   present in the file are not present in the list, i.e., they are `NULL`.
 #'
 #' @seealso
 #'  \describe{
@@ -251,6 +254,7 @@ buildForbiddenExp <- function(configurations, parameters)
 #' }
 #' 
 #' @author Manuel López-Ibáñez and Jérémie Dubois-Lacoste
+#' @md
 #' @export
 readScenario <- function(filename = "", scenario = list())
 {
@@ -262,7 +266,16 @@ readScenario <- function(filename = "", scenario = list())
       irace.error ("The scenario file ", shQuote(rfilename), " included from ",
                    shQuote(topfile), " does not exist.")
     }
-    source(rfilename, local = envir., chdir = TRUE)
+    handle.source.error <- function(e) {
+      irace.error("Reading scenario file ", shQuote(rfilename),
+                  " included from ", shQuote(topfile),
+                  " produced the following errors or warnings:\n",
+                  paste0(conditionMessage(e), collapse="\n"))
+      return(NULL)
+    }
+    withCallingHandlers(
+      tryCatch(source(rfilename, local = envir., chdir = TRUE),
+               error = handle.source.error, warning = handle.source.error))
   }
 
   # First find out which file...
@@ -284,7 +297,15 @@ readScenario <- function(filename = "", scenario = list())
     if (debug.level >= 1)
       cat ("# Reading scenario file", shQuote(filename), ".......")
     # chdir = TRUE to allow recursive sourcing.
-    source(filename, local = TRUE, chdir = TRUE)
+    handle.source.error <- function(e) {
+      irace.error("Reading scenario file ", shQuote(filename),
+                  " produced the following errors or warnings:\n",
+                  paste0(conditionMessage(e), collapse="\n"))
+      return(NULL)
+    }
+    withCallingHandlers(
+      tryCatch(source(filename, local = TRUE, chdir = TRUE),
+               error = handle.source.error, warning = handle.source.error))
     if (debug.level >= 1) cat (" done!\n")
   } else {
     irace.error ("The scenario file ", shQuote(filename), " does not exist.")
@@ -309,11 +330,10 @@ readScenario <- function(filename = "", scenario = list())
 
 #' Check and correct the given scenario
 #'
-#' \code{checkScenario} takes a (possibly incomplete) scenario setup of
+#' `checkScenario` takes a (possibly incomplete) scenario setup of
 #' \pkg{irace}, checks for errors and transforms it into a valid scenario.
 #' 
-#' @param scenario A list where tagged elements correspond to scenario
-#' settings of \pkg{irace}.
+#' @template arg_scenario
 #' 
 #' @return The scenario received as a parameter, possibly corrected. Unset
 #' scenario settings are set to their default values.
@@ -334,6 +354,7 @@ readScenario <- function(filename = "", scenario = list())
 #' }
 #' 
 #' @author Manuel López-Ibáñez and Jérémie Dubois-Lacoste
+#' @md
 #' @export
 ## FIXME: This function should only do checks and return TRUE/FALSE. There
 ## should be other function that does the various transformations.
@@ -343,6 +364,25 @@ checkScenario <- function(scenario = defaultScenario())
   {
     return(paste0("'", name, "' (", .irace.params.def[name, "long"], ")"))
   }
+
+  as.boolean.param <- function(x, name)
+  {
+    x <- as.integer(x)
+    if (is.na (x) || (x != 0 && x != 1)) {
+      irace.error (quote.param(name), " must be either 0 or 1.")
+    }
+    return(as.logical(x))
+  }
+
+  check.valid.param <- function(x, valid)
+  {
+    if (scenario[[x]] %!in% valid) {
+      irace.error ("Invalid value '", scenario[[x]], "' of ",
+                   quote.param(x), ", valid values are: ",
+                   paste0(valid, collapse = ", "))
+    }
+  }
+  
   
   # Fill possible unset (NULL) with default settings.
   scenario <- defaultScenario (scenario)
@@ -351,16 +391,22 @@ checkScenario <- function(scenario = defaultScenario())
   dups <- anyDuplicated(names(scenario))
   if (dups > 0)
     irace.error("scenario contains duplicated entries: ", names(scenario)[dups])
-  
+
+  # Boolean control parameters.
+  boolParams <- .irace.params.def[.irace.params.def[, "type"] == "b", "name"]
+  for (p in boolParams) {
+    scenario[[p]] <- as.boolean.param (scenario[[p]], p)
+  }
+
   ## Check that everything is fine with external parameters
   # Check that the files exist and are readable.
   scenario$parameterFile <- path.rel2abs(scenario$parameterFile)
-  # We don't check parameterFile here because the user may give the
-  # parameters explicitly. And it is checked in readParameters anyway.
+  # We don't read parameterFile here because the user may give the parameters
+  # explicitly.  And it is validated in readParameters anyway.
   scenario$execDir <- path.rel2abs(scenario$execDir)
   file.check (scenario$execDir, isdir = TRUE,
               text = paste0("execution directory ", quote.param("execDir")))
-  
+  options(.irace.execdir = scenario$execDir)
   if (!is.null.or.empty(scenario$logFile)) {
     scenario$logFile <- path.rel2abs(scenario$logFile, cwd = scenario$execDir)
     file.check(scenario$logFile, writeable = TRUE,
@@ -408,9 +454,10 @@ checkScenario <- function(scenario = defaultScenario())
       scenario$targetRunner <- path.rel2abs(scenario$targetRunner)
       file.check (scenario$targetRunner, executable = TRUE,
                   text = paste0("target runner ", quote.param("targetRunner")))
-      .irace$target.runner <- target.runner.default
+      .irace$target.runner <- if (scenario$aclib)
+                                target.runner.aclib else target.runner.default
     } else {
-      irace.error("'targetRunner' must be a function or an executable program")
+      irace.error(quote.param ('targetRunner'), " must be a function or an executable program")
     }
   }
 
@@ -466,8 +513,7 @@ checkScenario <- function(scenario = defaultScenario())
   if (!is.null(scenario$testInstances)
       && is.null(names(scenario$testInstances))) {
     # Create unique IDs for testInstances
-    names (scenario$testInstances) <-
-      paste0(1:length(scenario$testInstances), "t")
+    names(scenario$testInstances) <- paste0(1:length(scenario$testInstances), "t")
   }
   
   # Configurations file
@@ -475,8 +521,16 @@ checkScenario <- function(scenario = defaultScenario())
     scenario$configurationsFile <- path.rel2abs(scenario$configurationsFile)
     file.check (scenario$configurationsFile, readable = TRUE,
                 text = "configurations file")
+    # We cannot read the configurations here because we need the parameters.
+    # FIXME: We should have the parameters inside scenario.
   }
-
+  
+  if (is.null.or.empty(scenario$initConfigurations)) {
+    scenario$initConfigurations <- NULL
+  } else if (!is.data.frame(scenario$initConfigurations) && !is.matrix(scenario$initConfigurations)) {
+    irace.error("if given, initConfigurations must be a matrix or data.frame")
+  }
+  
   # This prevents loading the file two times and overriding forbiddenExps if
   # the user specified them explicitly.
   if (is.null.or.empty(scenario$forbiddenExps)
@@ -527,6 +581,12 @@ checkScenario <- function(scenario = defaultScenario())
     }
     scenario$mu <- scenario$firstTest
   }
+
+  # AClib benchmarks use 15 digits
+  if (scenario$aclib)
+    scenario$digits <- 15
+  if (scenario$digits > 15 || scenario$digits <= 0)
+    irace.error (quote.param ("digits"), " must be within [1,15].")
   
   # Real [0, 1] control parameters
   realParams <- .irace.params.def[.irace.params.def[, "type"] == "r", "name"]
@@ -567,20 +627,6 @@ checkScenario <- function(scenario = defaultScenario())
     scenario$softRestartThreshold <- 10^(- scenario$digits)
   }
   
-  # Boolean control parameters
-  as.boolean.param <- function(x, name)
-  {
-    x <- as.integer(x)
-    if (is.na (x) || (x != 0 && x != 1)) {
-      irace.error (quote.param(name), " must be either 0 or 1.")
-    }
-    return(as.logical(x))
-  }
-  boolParams <- .irace.params.def[.irace.params.def[, "type"] == "b", "name"]
-  for (p in boolParams) {
-    scenario[[p]] <- as.boolean.param (scenario[[p]], p)
-  }
-
   if (scenario$deterministic &&
       scenario$firstTest > length(scenario$instances)) {
     irace.error("When deterministic == TRUE, the number of instances (",
@@ -598,13 +644,7 @@ checkScenario <- function(scenario = defaultScenario())
   if (scenario$batchmode != 0) {
     scenario$batchmode <- tolower(scenario$batchmode)
     # FIXME: We should encode options in the large table in main.R
-    valid.batchmode <- c("sge", "pbs", "torque", "slurm")
-    if (!(scenario$batchmode %in% valid.batchmode)) {
-      irace.error ("Invalid value '", scenario$batchmode,
-                   "' of ", quote.param("batchmode"),
-                   ", valid values are: ",
-                   paste0(valid.batchmode, collapse = ", "))
-    }
+    check.valid.param("batchmode", valid = c("sge", "pbs", "torque", "slurm"))
   }
   # Currently batchmode requires a targetEvaluator
   if (scenario$batchmode != 0 && is.null(scenario$targetEvaluator)) {
@@ -623,26 +663,19 @@ checkScenario <- function(scenario = defaultScenario())
     if (scenario$boundMax <= 0) 
       irace.error("When capping == TRUE, boundMax (", scenario$boundMax,
                   ") must be > 0")
-    valid.types <- c("median", "mean", "worst", "best")
-    if (!(scenario$cappingType %in% valid.types)) {
-      irace.error ("Invalid value '", scenario$cappingType,
-                   "' of ", quote.param("cappingType"),
-                   ", valid values are: ",
-                   paste0(valid.types, collapse = ", "))
-    }
-    
-    valid.types <- c("instance", "candidate")
-    if (!(scenario$boundType %in% valid.types)) {
-      irace.error ("Invalid value '", scenario$boundType,
-                   "' of ", quote.param("boundType"),
-                   ", valid values are: ",
-                   paste0(valid.types, collapse = ", "))
-    }    
-    
+    check.valid.param("cappingType",
+                      valid = c("median", "mean", "worst", "best"))
+    check.valid.param("boundType", valid = c("instance", "candidate"))
+        
     if (scenario$boundPar < 1)
       irace.error("Invalid value boundPar (", scenario$boundPar,
                   ") must be >= 1")
+  } else { # no capping
+    if (scenario$boundMax <= 0 || is.na(scenario$boundMax))
+      scenario$boundMax <- NULL
   }
+
+
   
   if (is.null.or.empty(scenario$testType) || 
       is.null.or.na(scenario$testType) || 
@@ -661,26 +694,15 @@ checkScenario <- function(scenario = defaultScenario())
            "t.holm" = "t.holm",
            "t-test-bonferroni" =, # Fall-through,
            "t.bonferroni" = "t.bonferroni",
-           irace.error ("Invalid value '", scenario$testType,
-                        "' of ", quote.param("testType"),
-                        ", valid values are: ",
-                        "F-test, t-test, t-test-holm, t-test-bonferroni"))
-                        
-
+           check.valid.param("testType",
+                             c("F-test, t-test, t-test-holm, t-test-bonferroni")))
+  
   return (scenario)
 }
 
-print.instances <- function(param, value)
-{
-  cat (param, "= \"")
-  cat (value, sep = ", ")
-  cat ("\"\n")
-}
-
 #' Prints the given scenario
 #'
-#' @param scenario A list where tagged elements correspond to scenario
-#'    settings of \pkg{irace}.
+#' @template arg_scenario
 #' 
 #' @seealso
 #'  \describe{
@@ -691,20 +713,17 @@ print.instances <- function(param, value)
 #' }
 #' 
 #' @author Manuel López-Ibáñez and Jérémie Dubois-Lacoste
+#' @md
 #' @export
 printScenario <- function(scenario)
 {
   cat("## irace scenario:\n")
   for (param in .irace.params.names) {
-    
-    # Special case for instances
-    if (param == "instances" || param == "testInstances") {
-      print.instances (param, scenario[[param]])
-    } else {# All other parameters (no vector, but can be functions)
-      # FIXME: Perhaps deparse() is not the right way to do this?
-      # FIXME: Perhaps toString() ?
-      cat(param, "=", deparse(scenario[[param]]), "\n")
-    }
+    if (param == "forbiddenExps")
+      extra <- paste0(" = expression(", paste0(collapse=", ",
+                                  sapply(scenario[[param]], attr, "source")), ")")
+    else extra <- ""
+    cat(param, " = ", deparse(scenario[[param]]), extra, "\n", sep = "")
   }
   cat("## end of irace scenario\n")
 }
@@ -713,8 +732,7 @@ printScenario <- function(scenario)
 #'
 #' Return scenario with default values.
 #' 
-#' @param scenario A list where tagged elements correspond to scenario
-#' settings of \pkg{irace}.
+#' @template arg_scenario
 #' 
 #' @return A list indexed by the \pkg{irace} parameter names,
 #' containing the default values for each parameter, except for those
@@ -731,6 +749,7 @@ printScenario <- function(scenario)
 #'      \item{\code{seed}}{Seed of the random number generator (by default, generate a random seed). (Default: \code{NA})}
 #'      \item{\code{repairConfiguration}}{User-defined R function that takes a configuration generated by irace and repairs it. (Default: \code{""})}
 #'      \item{\code{postselection}}{Percentage of the configuration budget used to perform a postselection race of the best configurations of each iteration after the execution of irace. (Default: \code{0})}
+#'      \item{\code{aclib}}{Enable/disable AClib mode. This option enables compatibility with GenericWrapper4AC as targetRunner script. (Default: \code{0})}
 #'    }
 #'  \item Elitist \code{irace}:
 #'    \describe{
@@ -771,7 +790,8 @@ printScenario <- function(scenario)
 #'    }
 #'  \item Initial configurations:
 #'    \describe{
-#'      \item{\code{configurationsFile}}{File that contains a set of initial configurations. If empty or \code{NULL}, all initial configurations are randomly generated. (Default: \code{""})}
+#'      \item{\code{initConfigurations}}{Data frame describing initial configurations (usually read from a file using \code{readConfigurations}). (Default: \code{""})}
+#'      \item{\code{configurationsFile}}{File that contains a table of initial configurations. If empty or \code{NULL}, all initial configurations are randomly generated. (Default: \code{""})}
 #'    }
 #'  \item Training instances:
 #'    \describe{
@@ -884,21 +904,23 @@ checkTargetFiles <- function(scenario, parameters)
 {
   result <- TRUE
   ## Create two random configurations
-  conf.id <- c("testConfig1","testConfig2")
+  conf.id <- c("testConfig1", "testConfig2")
   configurations <- sampleUniform(parameters, length(conf.id),
                                   digits = scenario$digits,
                                   forbidden = scenario$forbiddenExps,
                                   repair = scenario$repairConfiguration)
   configurations <- cbind (.ID. = conf.id, configurations)
 
-  bounds <- if (scenario$capping)
-              rep(scenario$boundMax, length(configurations)) else NULL
+  bounds <- rep(scenario$boundMax, nrow(configurations))
 
-  experiments <- createExperimentList(configurations, parameters,
-                                      scenario$instances[1],
-                                      instances.ID = "instance1",
-                                      seeds = 1234567, scenario,
-                                      bounds = bounds)
+  instances.ID <- if (scenario$sampleInstances)
+                    sample.int(length(scenario$instances), 1) else 1
+  experiments <- createExperimentList(
+    configurations, parameters, instances = scenario$instances,
+    instances.ID = instances.ID, seeds = 1234567, scenario, bounds = bounds)
+
+  startParallel(scenario)
+  on.exit(stopParallel(), add = TRUE)
 
   # FIXME: Create a function try.call(err.msg,warn.msg, fun, ...)
   # Executing targetRunner
@@ -919,7 +941,7 @@ checkTargetFiles <- function(scenario, parameters)
 
   if (scenario$debugLevel >= 1) {
     cat ("# targetRunner returned:\n")
-    print(output)
+    print(output, digits = 15)
   }
   
   irace.assert(is.null(scenario$targetEvaluator) == is.null(.irace$target.evaluator))
@@ -941,7 +963,7 @@ checkTargetFiles <- function(scenario, parameters)
                    invokeRestart("muffleWarning")})
     if (scenario$debugLevel >= 1) {
       cat ("# targetEvaluator returned:\n")
-      print(output)
+      print(output, digits = 15)
     }
   }
   return(result)
