@@ -12,10 +12,6 @@ irace.reload.debug <- function(package = "irace")
 
 .irace.prefix <- "== irace == "
 
-.irace.bug.report <-
-  paste("An unexpected condition occurred.",
-        "Please report this bug to the authors of the irace package <http://iridia.ulb.ac.be/irace>")
-
 irace.print.memUsed <- function(objects)
 {
   object.size.kb <- function (name, envir) {
@@ -42,6 +38,13 @@ irace.print.memUsed <- function(objects)
   cat(sep="", sprintf("%30s : %17.1f Mb\n", "gc", sum(gc()[,2])))
 }
 
+# Print a user-level warning message, when the calling context
+# cannot help the user to understand why the program failed.
+irace.warning <- function(...)
+{
+  cat(sep="", "WARNING: ", ..., "\n")
+}
+
 # Print a user-level fatal error message, when the calling context
 # cannot help the user to understand why the program failed.
 irace.error <- function(...)
@@ -75,30 +78,35 @@ irace.dump.frames <- function()
 # Print an internal fatal error message that signals a bug in irace.
 irace.internal.error <- function(...)
 {
-  traceback(1)
+  .irace.bug.report <-
+    paste0("An unexpected condition occurred. ",
+           "Please report this bug to the authors of the irace package <http://iridia.ulb.ac.be/irace>")
+
   op <- options(warning.length = 8170,
                 error = if (interactive()) utils::recover
                         else irace.dump.frames)
   on.exit(options(op))
+  # 6 to not show anything below irace.assert()
+  bt <- capture.output(traceback(6))
   warnings()
-  stop (.irace.prefix, ..., "\n", .irace.bug.report, call. = TRUE)
+  stop (.irace.prefix, paste0(..., collapse = "\n"),
+        paste0(bt, collapse= "\n"), "\n",
+        .irace.prefix, "\n", .irace.bug.report, call. = FALSE)
   invisible()
 }
 
-irace.assert <- function(exp)
+irace.assert <- function(exp, eval.after = NULL)
 {
-  if (exp) return(invisible())
-  mc <- match.call()[[2]]
-  msg <- paste0(deparse(mc), " is not TRUE\n", .irace.bug.report)
   # FIXME: It would be great if we could save into a file the state of
   # the function that called this one.
-  traceback(1)
-  op <- options(warning.length = 8170,
-                error = if (interactive()) utils::recover
-                        else irace.dump.frames)
-  on.exit(options(op))
-  warnings()
-  stop (msg)
+  if (exp) return(invisible())
+  mc <- match.call()[[2]]
+  msg <- paste0(deparse(mc), " is not TRUE\n")
+  if (!is.null(eval.after)) {
+    msg.after <- eval.parent(capture.output(eval.after))
+    msg <- paste0(msg, "\n", msg.after)
+  }
+  irace.internal.error(msg)
   invisible()
 }
 
@@ -181,15 +189,19 @@ is.na.nowarn <- function(x)
   length(x) == 1 && suppressWarnings(is.na(x))
 }
 
+is.na.or.empty <- function(x)
+{
+  (length(x) == 0) || is.na.nowarn(x)
+}
+
 is.null.or.na <- function(x)
 {
   is.null(x) || is.na.nowarn(x)
 }
 
-# FIXME: This should also return TRUE when length(x) == 0
 is.null.or.empty <- function(x)
 {
-  is.null(x) || (!suppressWarnings(is.na(x)) && length(x) == 1 && is.character(x) && x == "")
+  (length(x) == 0) || (length(x) == 1 && !suppressWarnings(is.na(x)) && is.character(x) && x == "")
 }
 
 is.function.name <- function(FUN)
@@ -197,19 +209,15 @@ is.function.name <- function(FUN)
   # FIXME: Is there a simpler way to do this check?
   is.function(FUN) ||
     (!is.null(FUN) && !is.na(FUN) && as.character(FUN) != "" &&
-     !is.null(mget(as.character(FUN), envir = as.environment(-1),
-                   mode = "function", ifnotfound = list(NULL),
-                   inherits = TRUE)[[1]]))
+     !is.null(get.function(FUN)))
 }
 
 get.function <- function(FUN)
 {
-  irace.assert(is.function.name(FUN))
   if (is.function(FUN)) return(FUN)
-  # FIXME: Is there a simpler way to do this?
-  return(mget(FUN, envir = as.environment(-1),
-              mode = "function", ifnotfound = list(NULL),
-              inherits = TRUE)[[1]])
+  FUN <- dynGet(as.character(FUN), ifnotfound = NULL, inherits = TRUE)
+  if (is.function(FUN)) return(FUN)
+  return (NULL)
 }
 
 is.bytecode <- function(x) typeof(x) == "bytecode"
@@ -220,6 +228,17 @@ bytecompile <- function(x)
   return(compiler::cmpfun(x))
 }
 
+# FIXME: Use stringr function and replace this function
+str_sub <- function(x, start=0, stop=nchar(x))
+{
+  negs <- start < 0
+  if (any(negs)) start[negs] <- nchar(x[negs]) + 1  - start[negs]
+
+  negs <- stop < 0
+  if (any(negs)) stop[negs] <- nchar(x[negs]) + 1  - stop[negs]
+  return(substr(x, start, stop))
+}
+
 strcat <- function(...)
 {
   do.call(paste0, args = list(..., collapse = NULL))
@@ -336,6 +355,36 @@ path.rel2abs <- function (path, cwd = getwd())
   return (irace.normalize.path(path))
 }
 
+#' Update filesystem paths of a scenario consistently.
+#'
+#' This function should be used to change the filesystem paths stored in a
+#' scenario object. Useful when moving a scenario from one computer to another.
+#'
+#' @template arg_scenario
+#' @param from character string containing a regular expression (or character
+#'   string for \code{fixed = TRUE}) to be matched.
+#' @param to the replacement string.character string. For \code{fixed = FALSE}
+#'   this can include backreferences \code{"\1"} to \code{"\9"} to
+#'   parenthesized subexpressions of \code{from}.
+#' @param fixed logical.  If \code{TRUE}, \code{from} is a string to be matched
+#'   as is.
+#' @return The updated scenario
+#' @examples
+#' \dontrun{
+#' scenario <- readScenario(filename = "scenario.txt")
+#' scenario <- scenario.update.paths(scenario, from = "/home/manuel/", to = "/home/leslie")
+#' }
+#' @seealso \code{\link[base]{grep}}
+#' @export
+scenario.update.paths <- function(scenario, from, to, fixed = TRUE)
+{
+  pathParams <- .irace.params.def[.irace.params.def[, "type"] == "p", "name"]
+  # Only consider the ones that actually appear in scenario.
+  pathParams <- intersect(pathParams, names(scenario))
+  scenario[pathParams] <- lapply(scenario[pathParams], sub, pattern = from, replacement = to, fixed = fixed)
+  return(scenario)
+}
+
 # This function is used to trim potentially large strings for printing, since
 # the maximum error/warning length is 8170 characters (R 3.0.2)
 strlimit <- function(str, limit = 5000)
@@ -423,6 +472,8 @@ merge.matrix <- function(x, y)
                     dimnames = list(new.rows, colnames(x))))
   # Update
   x[rownames(y), colnames(y)] <- y
+  # There must be a non-NA entry for each instance.
+  irace.assert(all(apply(!is.na(x), 1, any)))
   return(x)
 }
 
@@ -452,7 +503,7 @@ extractElites <- function(configurations, nbElites)
 #' the configurations, to output only the values for the parameters
 #' of the configuration without data possibly useless to the user.
 #'   
-#' @param configurations A matrix containg the configurations, one per row.
+#' @template arg_configurations
 #' 
 #' @return The same matrix without the "metadata".
 #'    
@@ -472,7 +523,7 @@ removeConfigurationsMetaData <- function(configurations)
 
 #' Print configurations as a data frame
 #' 
-#' @param configurations a data frame containing the configurations (one per row).
+#' @template arg_configurations
 #' @param metadata A Boolean specifying whether to print the metadata or
 #' not. The metadata are data for the configurations (additionally to the
 #' value of each parameter) used by \pkg{irace}.
@@ -490,7 +541,7 @@ configurations.print <- function(configurations, metadata = FALSE)
   if (!metadata) {
     configurations <- removeConfigurationsMetaData(configurations)
   } 
-  print(as.data.frame(configurations, stringsAsFactors = FALSE))
+  print(as.data.frame(configurations, stringsAsFactors = FALSE), digits = 15)
 }
 
 #' Print configurations as command-line strings.
@@ -498,9 +549,8 @@ configurations.print <- function(configurations, metadata = FALSE)
 #' Prints configurations after converting them into a representation for the
 #' command-line.
 #' 
-#' @param configurations a data frame containing the configurations (one per row).
-#' @param parameters A data structure similar to that provided by the 
-#' \code{\link{readParameters}} function.
+#' @template arg_configurations
+#' @template arg_parameters
 #' 
 #' @return None.
 #'
@@ -680,6 +730,8 @@ runcommand <- function(command, args, id, debugLevel)
                err <<- c(err, paste(conditionMessage(w), collapse="\n"))
                invokeRestart("muffleWarning")
              })
+  if (is.null(output))
+    output <- ""
   # If the command could not be run an R error is generated.  If ‘command’
   # runs but gives a non-zero exit status this will be reported with a
   # warning and in the attribute ‘"status"’ of the result: an attribute
@@ -687,13 +739,13 @@ runcommand <- function(command, args, id, debugLevel)
   if (!is.null(err)) {
     err <- paste(err, collapse = "\n")
     if (!is.null(attr(output, "errmsg")))
-      output <- paste(sep = "\n", attr(output, "errmsg"))
+      err <- paste(sep = "\n", err, attr(output, "errmsg"))
     if (debugLevel >= 2L)
       irace.note ("ERROR (", id, "): ", err, "\n")
     return(list(output = output, error = err))
   }
   if (debugLevel >= 2L) {
-    irace.note ("DONE (", id, ") Elapsed: ",
+    irace.note ("DONE (", id, ") Elapsed wall-clock seconds: ",
                 formatC(proc.time()["elapsed"] - elapsed,
                         format = "f", digits = 2), "\n")
   }
@@ -732,3 +784,13 @@ is.file.extension <- function(filename, ext)
 
 # Same as !(x %in% table)
 "%!in%" <- function (x, table) match(x, table, nomatch = 0L) == 0L
+
+irace_save_logfile <- function(iraceResults, scenario)
+{
+  if (is.null.or.empty(scenario$logFile)) return(invisible())
+  cwd <- setwd(scenario$execDir)
+  # FIXME: Use saveRDS
+  # FIXME: Bump to version=3 when we bump the minimum R version to >=3.6
+  save (iraceResults, file = scenario$logFile, version = 2)
+  setwd (cwd)
+}
